import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Account } from '../entities/account.entity';

@Injectable()
export class AccountRepository {
  constructor(@InjectRepository(Account) private readonly accountRepository: Repository<Account>) {}

  createAccount(accountId: string): Promise<Account> {
    return this.accountRepository.save(
      this.accountRepository.create({
        accountReference: accountId,
        balance: 0,
        currency: '682',
      }),
    );
  }

  getAccountByReferenceNumber(accountReference: string): Promise<Account | null> {
    return this.accountRepository.findOne({
      where: { accountReference },
      relations: ['cards'],
    });
  }

  topUpAccountBalance(accountReference: string, amount: number) {
    return this.accountRepository.increment({ accountReference }, 'balance', amount);
  }

  decreaseAccountBalance(accountReference: string, amount: number) {
    return this.accountRepository.decrement({ accountReference }, 'balance', amount);
  }
}
