import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { CreateApplicationResponse } from '~/common/modules/neoleap/dtos/response';
import { Card } from '../entities';
import { CardColors, CardIssuers, CardScheme, CardStatus, CustomerType } from '../enums';

@Injectable()
export class CardRepository {
  constructor(@InjectRepository(Card) private readonly cardRepository: Repository<Card>) {}

  createCard(customerId: string, accountId: string, card: CreateApplicationResponse): Promise<Card> {
    return this.cardRepository.save(
      this.cardRepository.create({
        customerId: customerId,
        expiry: card.expiryDate,
        cardReference: card.cardId,
        customerType: CustomerType.PARENT,
        firstSixDigits: card.firstSixDigits,
        lastFourDigits: card.lastFourDigits,
        color: CardColors.BLUE,
        status: CardStatus.ACTIVE,
        scheme: CardScheme.VISA,
        issuer: CardIssuers.NEOLEAP,
        accountId: accountId,
      }),
    );
  }

  getCardById(id: string): Promise<Card | null> {
    return this.cardRepository.findOne({ where: { id } });
  }

  getCardByReferenceNumber(referenceNumber: string): Promise<Card | null> {
    return this.cardRepository.findOne({ where: { cardReference: referenceNumber }, relations: ['account'] });
  }
}
