import { Controller, Get } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { ApiTags } from '@nestjs/swagger';
import { CardService } from '~/card/services';
import { ApiDataResponse } from '~/core/decorators';
import { ResponseFactory } from '~/core/utils';
import { Customer } from '~/customer/entities';
import { CustomerService } from '~/customer/services';
import { CreateApplicationResponse, InquireApplicationResponse } from '../dtos/response';
import { NeoLeapService } from '../services/neoleap.service';

@Controller('neotest')
@ApiTags('Neoleap Test API , for testing purposes only, will be removed in production')
export class NeoTestController {
  constructor(
    private readonly neoleapService: NeoLeapService,
    private readonly customerService: CustomerService,
    private readonly cardService: CardService,
    private readonly configService: ConfigService,
  ) {}

  @Get('inquire-application')
  @ApiDataResponse(InquireApplicationResponse)
  async inquireApplication() {
    const data = await this.neoleapService.inquireApplication('15');
    return ResponseFactory.data(data);
  }

  @Get('create-application')
  @ApiDataResponse(CreateApplicationResponse)
  async createApplication() {
    const customer = await this.customerService.findCustomerById(
      this.configService.get<string>('MOCK_CUSTOMER_ID', 'ff462af1-1e0c-4216-8865-738e5b525ac1'),
    );
    const data = await this.neoleapService.createApplication(customer as Customer);
    await this.cardService.createCard(customer.id, data);
    return ResponseFactory.data(data);
  }
}
